package com.owlmaddie;

import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.networking.v1.ServerPlayNetworking;
import net.minecraft.entity.Entity;
import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.effect.StatusEffectInstance;
import net.minecraft.entity.effect.StatusEffects;
import net.minecraft.util.Identifier;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerWorldEvents;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class ModInit implements ModInitializer {
    public static final Logger LOGGER = LoggerFactory.getLogger("mobgpt");
	public static final Identifier PACKET_C2S_GREETING = new Identifier("mobgpt", "packet_c2s_greeting");
	public static final Identifier PACKET_C2S_READ_NEXT = new Identifier("mobgpt", "packet_c2s_read_next");

	@Override
	public void onInitialize() {
		// This code runs as soon as Minecraft is in a mod-load-ready state.
		// However, some things (like resources) may still be uninitialized.
		// Proceed with mild caution.

		// Handle packet for Greeting
		ServerPlayNetworking.registerGlobalReceiver(PACKET_C2S_GREETING, (server, player, handler, buf, responseSender) -> {
			int entityId = buf.readInt();

			// Ensure that the task is synced with the server thread
			server.execute(() -> {
				// Your logic here, e.g., handle the entity click
				Entity entity = player.getServerWorld().getEntityById(entityId);
				if (entity != null) {
					// Perform action with the clicked entity
					// Slow entity
					SlowEntity((LivingEntity) entity, 3.5F);

					LOGGER.info("Generate greeting for: " + entity.getType().toString());
					ChatDataManager.EntityChatData chatData = ChatDataManager.getInstance().getOrCreateChatData(entityId);
					chatData.generateGreeting();
				}
			});
		});

		// Handle packet for reading lines of message
		ServerPlayNetworking.registerGlobalReceiver(PACKET_C2S_READ_NEXT, (server, player, handler, buf, responseSender) -> {
			int entityId = buf.readInt();
			int lineNumber = buf.readInt();

			// Ensure that the task is synced with the server thread
			server.execute(() -> {
				// Your logic here, e.g., handle the entity click
				Entity entity = player.getServerWorld().getEntityById(entityId);
				if (entity != null) {
					// Perform action with the clicked entity
					// Slow entity
					SlowEntity((LivingEntity) entity, 3.5F);

					LOGGER.info("Increment read lines to " + lineNumber + " for: " + entity.getType().toString());
					ChatDataManager.EntityChatData chatData = ChatDataManager.getInstance().getOrCreateChatData(entityId);
					chatData.setLineNumber(lineNumber);
				}
			});
		});

		ServerWorldEvents.LOAD.register((server, world) -> {
			// Load chat data...
			LOGGER.info("LOAD chat data from NBT: " + world.getRegistryKey().getValue());
		});
		ServerWorldEvents.UNLOAD.register((server, world) -> {
			// Save chat data...
			LOGGER.info("SAVE chat data to NBT: " + world.getRegistryKey().getValue());
		});

		LOGGER.info("MobGPT Initialized!");
	}

	public void SlowEntity(LivingEntity entity, float numSeconds) {
		// Slow the entity temporarily (so they don't run away)
		// Apply a slowness effect with a high amplifier for a short duration
		// (Amplifier value must be between 0 and 127)
		float TPS = 20F; // ticks per second
		StatusEffectInstance slowness = new StatusEffectInstance(StatusEffects.SLOWNESS, Math.round(numSeconds * TPS),
				127, false, false);
		entity.addStatusEffect(slowness);
	}
}